@@ -65,8 +65,8 @@ def _skip_exchange_algorithm_unsupported(backend, algorithm, curve):
         algorithm, curve
     ):
         pytest.skip(
-            "Exchange algorithm is not supported by this backend {0}".format(
-                backend
+            "Exchange with {0} curve is not supported by {1}".format(
+                curve.name, backend
             )
         )
 
@@ -142,43 +142,26 @@ def test_ec_numbers():
     assert numbers.public_numbers.y == 3
     assert isinstance(numbers.public_numbers.curve, DummyCurve)
 
+
+@pytest.mark.parametrize(
+    ("private_value", "x", "y", "curve"),
+    [
+        (None, 2, 3, DummyCurve()),
+        (1, None, 3, DummyCurve()),
+        (1, 2, None, DummyCurve()),
+        (1, 2, 3, None),
+    ]
+)
+def test_invalid_ec_numbers_args(private_value, x, y, curve):
     with pytest.raises(TypeError):
         ec.EllipticCurvePrivateNumbers(
-            None,
-            ec.EllipticCurvePublicNumbers(
-                2, 3, DummyCurve()
-            )
+            private_value, ec.EllipticCurvePublicNumbers(x, y, curve)
         )
 
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                None, 3, DummyCurve()
-            )
-        )
 
+def test_invalid_private_numbers_public_numbers():
     with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                2, None, DummyCurve()
-            )
-        )
-
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            ec.EllipticCurvePublicNumbers(
-                2, 3, None
-            )
-        )
-
-    with pytest.raises(TypeError):
-        ec.EllipticCurvePrivateNumbers(
-            1,
-            None
-        )
+        ec.EllipticCurvePrivateNumbers(1, None)
 
 
 def test_encode_point():
@@ -395,7 +378,7 @@ class TestECDSAVectors(object):
 
         with raises_unsupported_algorithm(
             exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
+        ), pytest.warns(CryptographyDeprecationWarning):
             key.signer(DummySignatureAlgorithm())
 
         with raises_unsupported_algorithm(
@@ -405,7 +388,7 @@ class TestECDSAVectors(object):
 
         with raises_unsupported_algorithm(
             exceptions._Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM
-        ):
+        ), pytest.warns(CryptographyDeprecationWarning):
             key.public_key().verifier(b"", DummySignatureAlgorithm())
 
         with raises_unsupported_algorithm(
@@ -563,9 +546,7 @@ class TestECDSAVectors(object):
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(message, algorithm)
         public_key = private_key.public_key()
-        verifier = public_key.verifier(signature, algorithm)
-        verifier.update(message)
-        verifier.verify()
+        public_key.verify(signature, message, algorithm)
 
     def test_sign_prehashed(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
@@ -577,9 +558,7 @@ class TestECDSAVectors(object):
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         signature = private_key.sign(data, algorithm)
         public_key = private_key.public_key()
-        verifier = public_key.verifier(signature, ec.ECDSA(hashes.SHA1()))
-        verifier.update(message)
-        verifier.verify()
+        public_key.verify(signature, message, ec.ECDSA(hashes.SHA1()))
 
     def test_sign_prehashed_digest_mismatch(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
@@ -597,9 +576,7 @@ class TestECDSAVectors(object):
         message = b"one little message"
         algorithm = ec.ECDSA(hashes.SHA1())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        signer = private_key.signer(algorithm)
-        signer.update(message)
-        signature = signer.finalize()
+        signature = private_key.sign(message, algorithm)
         public_key = private_key.public_key()
         public_key.verify(signature, message, algorithm)
 
@@ -608,9 +585,7 @@ class TestECDSAVectors(object):
         message = b"one little message"
         algorithm = ec.ECDSA(hashes.SHA1())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        signer = private_key.signer(algorithm)
-        signer.update(message)
-        signature = signer.finalize()
+        signature = private_key.sign(message, algorithm)
         h = hashes.Hash(hashes.SHA1(), backend)
         h.update(message)
         data = h.finalize()
@@ -635,14 +610,16 @@ class TestECDSAVectors(object):
     def test_prehashed_unsupported_in_signer_ctx(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError), \
+                pytest.warns(CryptographyDeprecationWarning):
             private_key.signer(ec.ECDSA(Prehashed(hashes.SHA1())))
 
     def test_prehashed_unsupported_in_verifier_ctx(self, backend):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         private_key = ec.generate_private_key(ec.SECP256R1(), backend)
         public_key = private_key.public_key()
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError), \
+                pytest.warns(CryptographyDeprecationWarning):
             public_key.verifier(
                 b"0" * 64,
                 ec.ECDSA(Prehashed(hashes.SHA1()))
@@ -1039,7 +1016,8 @@ class TestECDSAVerification(object):
         _skip_curve_unsupported(backend, ec.SECP256R1())
         key = ec.generate_private_key(ec.SECP256R1(), backend)
         public_key = key.public_key()
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeError), \
+                pytest.warns(CryptographyDeprecationWarning):
             public_key.verifier(1234, ec.ECDSA(hashes.SHA256()))
 
 
@@ -1068,9 +1046,9 @@ class TestECDH(object):
                 ec._CURVE_TYPES[vector['curve']]()
             )
         )
-        # Errno 5 and 6 indicates a bad public key, this doesn't test the ECDH
-        # code at all
-        if vector['fail'] and vector['errno'] in [5, 6]:
+        # Errno 5-7 indicates a bad public or private key, this doesn't test
+        # the ECDH code at all
+        if vector['fail'] and vector['errno'] in [5, 6, 7]:
             with pytest.raises(ValueError):
                 private_numbers.private_key(backend)
             return
@@ -1097,6 +1075,8 @@ class TestECDH(object):
         # At this point fail indicates that one of the underlying keys was
         # changed. This results in a non-matching derived key.
         if vector['fail']:
+            # Errno 8 indicates Z should be changed.
+            assert vector['errno'] == 8
             assert z != vector['Z']
         else:
             assert z == vector['Z']
